
/*
xpequ copyright (c) 2007-always Jan Panteltje

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "xpequ.h"


pthread_t audio_thread;
pthread_attr_t *attributes;

int stop_audio_flag;

double data_bytes_read;
double data_bytes_written;

FILE *ifptr;
FILE *ofptr;

complex_t wave[NUMSAMPLES * 2];
uint32_t map[XYPLOT_HEIGHT][XYPLOT_WIDTH / 2];
int cur_line;
int lines_per_channel;
int inbuf_size, outbuf_size;

/* alsa */
snd_pcm_t *playback_handle;
snd_pcm_hw_params_t *out_hw_params;

snd_pcm_t *capture_handle;
snd_pcm_hw_params_t *in_hw_params;
/* end alsa */


int input_init()
{
int a;
int tmp;
FILE *pptr;
char temp[512];
unsigned int b;

if(debug_flag)
	{
	fprintf(stderr, "input_init(): arg none, using audio_mode=%d audio_input_device_name=%s\n", audio_mode, audio_input_device_name);
	}

if(internet_mode == INTERNET_CLIENT)
	{
	if(internet_stream_flag) return 1;
	}

inbuf_size = abuf_size;
/* set mixer for record */

if(set_mixer_flag)
	{
	sprintf(temp, "cmix rcs igain on");
	pptr = popen(temp, "r");
	pclose(pptr);

	sprintf(temp, "cmix mic 100");	
	pptr = popen(temp, "r");
	pclose(pptr);

	sprintf(temp, "cmix igain 50");
	pptr = popen(temp, "r");
	pclose(pptr);

	sprintf(temp, "cmix vol 100");
	pptr = popen(temp, "r");
	pclose(pptr);
	} /* end if set_mixer_flag */

if(use_input_file_flag)
	{
	if(strcmp(input_filename, "-") == 0)
		{
		ifptr = fdopen(fileno(stdin), "r");
		}
	else
		{
		ifptr = fopen(input_filename, "r");
		}
	if(! ifptr)
		{
		fprintf(stderr, "xpequ: input_init(): could not open input file %s for read, aborting.\n", input_filename);
		
		/* allow user to change device in GUI */
		return 0;
		}	

	/* read wave header if not streaming */
	if(! input_stream_flag)
		{
		a = fread(audiobuf, sizeof(char), header_size, ifptr);
		if(a < header_size)
			{
			fprintf(stderr, "xpequ: could not read wave header for file %s, aborting.\n", input_filename);

			return 0;
			}

		if( (audiobuf[0] != 'R') || (audiobuf[1] != 'I') || (audiobuf[2] != 'F') || (audiobuf[3] != 'F') )
			{
			fprintf(stderr, "xpequ: input file %s is not a RIFF file, aborting.\n", input_filename);
	
			return 0;
			}

		if( (audiobuf[8] != 'W') || (audiobuf[9] != 'A') || (audiobuf[10] != 'V') || (audiobuf[11] != 'E') )
			{
			fprintf(stderr, "xpequ: input file %s is not a WAVE file, aborting.\n", input_filename);
	
			return 0;
			}

		if( (audiobuf[12] != 'f') || (audiobuf[13] != 'm') || (audiobuf[14] != 't') || (audiobuf[15] != ' ') )
			{
			fprintf(stderr, "xpequ: WARNING input file %s header does not contain 'fmt '.\n", input_filename);
			}

		in_header -> length =		audiobuf[ 4] | (audiobuf[ 5] << 8) | (audiobuf[6] << 16) | (audiobuf[7] << 24); 
		in_header -> length_chunk =	audiobuf[16] | (audiobuf[17] << 8) | (audiobuf[18] << 16) | (audiobuf[19] << 24);
		in_header -> format =		audiobuf[20] | (audiobuf[21] << 8);
		in_header -> modus =		audiobuf[22] | (audiobuf[23] << 8);
		in_header -> sample_fq =	audiobuf[24] | (audiobuf[25] << 8) | (audiobuf[26] << 16) | (audiobuf[27] << 24);
		in_header -> byte_p_sec =	audiobuf[28] | (audiobuf[29] << 8) | (audiobuf[30] << 16) | (audiobuf[31] << 24);
		in_header -> byte_p_spl =	audiobuf[32] | (audiobuf[33] << 8);
		in_header -> bit_p_spl =	audiobuf[34] | (audiobuf[35] << 8);
		in_header -> data_length =	audiobuf[40] | (audiobuf[41] << 8) | (audiobuf[42] << 16) | (audiobuf[43] << 24);

		if(debug_flag)
			{
			fprintf(stderr, "in_header -> length =%d\n", in_header -> length);
			fprintf(stderr, "in_header -> length_chunk =%d\n", in_header -> length_chunk);
			fprintf(stderr, "in_header -> format =%d\n", in_header -> format);
			fprintf(stderr, "in_header -> modus =%d\n", in_header -> modus);
			fprintf(stderr, "in_header -> sample_fq =%d\n", in_header -> sample_fq);
			fprintf(stderr, "in_header -> byte_p_sec =%d\n", in_header -> byte_p_sec);
			fprintf(stderr, "in_header -> byte_p_spl =%d\n", in_header -> byte_p_spl);
			fprintf(stderr, "in_header -> bit_p_spl =%d\n", in_header -> bit_p_spl);
			fprintf(stderr, "in_header -> data_length =%d\n", in_header -> data_length);
			fprintf(stderr, "\n");

			fprintf(stderr,\
			"xpequ: bits per sample=%d  sample frequency=%d\n", in_header -> bit_p_spl, in_header -> sample_fq);
			}

		if( (in_header -> bit_p_spl != 8) && (in_header -> bit_p_spl != 16) )
			{
			fprintf(stderr, "xpequ: unsupported bits per sample %d, supported is 8 or 16, aborting.\n", in_header -> bit_p_spl); 
			
			return 0;
			}

		/* some tests for correct and same format input files */

		if(in_header -> length_chunk != 16)
			{
			fprintf(stderr, "xpequ: input file %s incorrect header, length_chunk != 16, aborting.\n", input_filename);
	
			return 0;
			}

		if(in_header -> format != 1)
			{
			fprintf(stderr, "xpequ: input file %s is not a PCM file, aborting.\n", input_filename);
	
			return 0;
			}

		sample_size = in_header -> bit_p_spl; 
		audio_channels = in_header -> modus;
		input_filelength = in_header -> length;
		input_datalength = in_header -> data_length;

//		dsp_stereo = audio_channels - 1; //in_header -> modus - 1;
		dsp_speed = in_header -> sample_fq;
		} /* end if ! input_stream_flag */ 

	dsp_stereo = audio_channels - 1; //in_header -> modus - 1;
	
	milli_seconds_per_sample = 1000.0 / (double)dsp_speed;

	if(debug_flag)
		{
		fprintf(stderr,\
		"input_init(): audio_channels=%d sample_size=%d dsp_speed=%d dsp_stereo=%d\n", audio_channels, sample_size, dsp_speed, dsp_stereo);
		}

	} /* end if use_input_file_flag */
else	
	{
	if(audio_mode == USE_ALSA)
		{
		audio_channels = 2;
		a = snd_pcm_open(&capture_handle, audio_input_device_name, SND_PCM_STREAM_CAPTURE, 0);
		if(a < 0)
			{
			fprintf(stderr, "input_init(): cannot open audio device %s for read because %s\n", audio_output_device_name, snd_strerror(a));

			if(command_line_input_device_specified_flag) exit(1);
			
			return 0;
			}

		a = snd_pcm_hw_params_malloc(&in_hw_params);
		if(a < 0)
			{
			fprintf(stderr, "input_init(): cannot allocate hardware parameter structure(%s)\n", snd_strerror(a));

			return 0;
			}

		a = snd_pcm_hw_params_any(capture_handle, in_hw_params);
		if(a < 0)
			{
			fprintf(stderr, "input_init(): cannot initialize hardware parameterstructure(%s)\n", snd_strerror(a));

			return 0;
			}

		a = snd_pcm_hw_params_set_access(capture_handle, in_hw_params, SND_PCM_ACCESS_RW_INTERLEAVED);
		if(a < 0)
			{
			fprintf(stderr, "input_init(): cannot set access type(%s)\n", snd_strerror(a));

			return 0;
			}

		a = snd_pcm_hw_params_set_format(capture_handle, in_hw_params, SND_PCM_FORMAT_S16_LE);
		if(a < 0)
			{
			fprintf(stderr, "input_init(): cannot set sample format(%s)\n", snd_strerror(a));

			return 0;
			}

		b = (unsigned int)dsp_speed;
		a = snd_pcm_hw_params_set_rate_near(capture_handle, in_hw_params, &b, 0);
		if(a < 0)
			{
			fprintf(stderr, "input_init(): cannot set sample rate(%s)\n", snd_strerror(a));
			
			return 0;
			}

		b = (int)dsp_speed;

		a = snd_pcm_hw_params_set_channels(capture_handle, in_hw_params, audio_channels);
//		a = snd_pcm_hw_params_get_channels(in_hw_params); , &audio_channels);
		if(a < 0)
			{
			fprintf(stderr, "input_init(): cannot set channel count(%s)\n", snd_strerror(a));
			
			return 0;
			}

//		a = snd_pcm_hw_params_set_periods(capture_handle, in_hw_params, 8192, 0);
//		if(a < 0)
			{
//			fprintf(stderr, "input_init(): cannot set periods(%s)\n", snd_strerror(a));
			
//			return 0;
			}

		a = snd_pcm_hw_params_set_buffer_size(capture_handle, in_hw_params, 8192);
		if(a < 0)
			{
			fprintf(stderr, "input_init(): cannot set buffer_size(%s)\n", snd_strerror(a));
			
			return 0;
			}

		a = snd_pcm_hw_params(capture_handle, in_hw_params);
		if(a < 0)
			{
			fprintf(stderr, "input_init(): cannot set parameters(%s)\n", snd_strerror(a));

			return 0;
			}

		snd_pcm_hw_params_free(in_hw_params);
		
		a = snd_pcm_prepare(capture_handle);
		if(a < 0)
			{
			fprintf(stderr, "input_init(): cannot prepare audio interface for use(%s)\n", snd_strerror(a));

			return 0;
			}

		} /* end if USE_ALSA */
	else if(audio_mode == USE_OSS)
		{	
		audio_in_fd = open(audio_input_device_name, O_RDONLY, 0);
		if(audio_in_fd < 0)
			{
			fprintf(stderr, "input_init(): input_init(): could not open %s for read\n", audio_input_device_name);
		
			if(command_line_input_device_specified_flag) exit(1);

			return 0;
			}

		if(debug_flag)
			{	
			fprintf(stderr, "input_init(): audio_in_fd=%d\n", audio_in_fd);
			}
		
		tmp = sample_size;
		ioctl(audio_in_fd, SNDCTL_DSP_SAMPLESIZE, &sample_size);
		if(tmp != sample_size)
			{
			fprintf(stderr, "input_init(): unable to set the sample size\n");

			return 0;
			}

		/* request stereo */
		dsp_stereo = 1;
		if(ioctl(audio_in_fd, SNDCTL_DSP_STEREO, &dsp_stereo) == -1)
			{
			fprintf (stderr, "input_init(): unable to set mono/stereo\n");

			return 0;
			}

		if(dsp_stereo == 0) audio_channels = 1;
		if(dsp_stereo == 1) audio_channels = 2;

		if(debug_flag)
			{
			fprintf(stderr, "input_init(): dsp_stereo=%d audio_channels=%d\n", dsp_stereo, audio_channels);
			}
	
		if(ioctl(audio_in_fd, SNDCTL_DSP_SPEED, &dsp_speed) == -1)
			{
			fprintf (stderr, "input_init(): unable to set audio speed\n");

			return 0;
			}

		if(debug_flag)
			{
			fprintf (stderr, "Speed %d Hz ", dsp_speed);
			if(sample_size != 8) 
				{
				fprintf(stderr, "input_init(): %d bits per sample\n", sample_size);
				}
			}

		ioctl(audio_in_fd, SNDCTL_DSP_GETBLKSIZE, &inbuf_size);
		if(inbuf_size < 1)
			{
			fprintf(stderr, "input_init(): DSP_GETBLKSIZE failed, because ");

			perror ("GETBLKSIZE");
			return 0;
			}

		} /* end if read from /dev/dspX */
	} /* end if USE_OSS */

return 1;
} /* end function input_init */


int output_init()
{
int a;
int tmp;
unsigned int b;

if(debug_flag)
	{
	fprintf(stderr, "output_init(): arg none, using audio_mode=%d audio_output_device_name=%s\n", audio_mode, audio_output_device_name);
	}

/* this uses values as get by input_init() */

outbuf_size = abuf_size;

//if( (output_flag) || ( (internet_mode == INTERNET_CLIENT) && internet_stream_flag && output_flag) )
if(output_flag)
	{
	if(strcmp(output_filename, "-") == 0)
		{
		ofptr = fdopen(fileno(stdout), "w");
		}
	else
		{
		ofptr = fopen(output_filename, "w");
		}
	if(! ofptr)
		{
		fprintf(stderr, "xpequ: output_init(): could not open output file %s for read, aborting.\n", output_filename);

		return 0;
		}	
	}
else // output to device
	{
	if(audio_mode == USE_ALSA)
		{
		a = snd_pcm_open(&playback_handle, audio_output_device_name, SND_PCM_STREAM_PLAYBACK, 0);
		if(a < 0)
			{
			fprintf(stderr, "output_init(): cannot open audio device %s(%s)\n", audio_output_device_name, snd_strerror(a) );

			if(command_line_output_device_specified_flag) exit(1);

			return 0;
			}

		a = snd_pcm_hw_params_malloc(&out_hw_params);
		if(a < 0)
			{
			fprintf(stderr, "output_init(): cannot allocate hardware parameter structure(%s)\n", snd_strerror(a));

			return 0;
			}

		a = snd_pcm_hw_params_any(playback_handle, out_hw_params);
		if(a < 0)
			{
			fprintf(stderr, "output_init(): cannot initialize hardware parameter structure(%s)\n", snd_strerror(a));

			return 0;
			}

		a = snd_pcm_hw_params_set_access(playback_handle, out_hw_params, SND_PCM_ACCESS_RW_INTERLEAVED);
		if(a < 0)
			{
			fprintf(stderr, "output_init(): cannot set access type(%s)\n", snd_strerror(a));

			return 0;
			}

		a = snd_pcm_hw_params_set_format(playback_handle, out_hw_params, SND_PCM_FORMAT_S16_LE);
		if(a < 0)
			{
			fprintf(stderr, "output_init(): cannot set sample format(%s)\n", snd_strerror(a));

			return 0;
			}

		b = (unsigned int)dsp_speed;
		a = snd_pcm_hw_params_set_rate_near(playback_handle, out_hw_params, &b, 0);
		if(a < 0)
			{
			fprintf(stderr, "output_init(): cannot set sample rate(%s)\n", snd_strerror(a));

			return 0;
			}
		dsp_speed = (int)b;

		a = snd_pcm_hw_params_set_channels(playback_handle, out_hw_params, audio_channels);
		if(a < 0)
			{
			fprintf(stderr, "output_init(): cannot set channel count(%s)\n", snd_strerror(a));

			return 0;
			}

//		a = snd_pcm_hw_params_set_buffer_size(playback_handle, in_hw_params, 8192);
//		if(a < 0)
			{
//			fprintf(stderr, "output_init(): cannot set buffer_size(%s)\n", snd_strerror(a));
			
//			return 0;
			}

		a = snd_pcm_hw_params(playback_handle, out_hw_params);
		if(a < 0)		
			{
			fprintf(stderr, "output_init(): cannot set parameters(%s)\n", snd_strerror(a));

			return 0;
			}

		snd_pcm_hw_params_free(out_hw_params);

		a = snd_pcm_prepare(playback_handle);
		if(a < 0)
			{
			fprintf(stderr, "output_init(): cannot prepare audio interface for use(%s)\n", snd_strerror(a));

			return 0;
			}

		} /* end if USE_ALSA */
	else if(audio_mode == USE_OSS)
		{	
		audio_out_fd = open(audio_output_device_name, O_RDWR, 0);
		if(audio_out_fd < 0)
			{
			fprintf(stderr, "output_init(): could not open %s for write\n", audio_output_device_name);
		
			if(command_line_output_device_specified_flag) exit(1);

			return 0;
			}

		if(debug_flag)
			{	
			fprintf(stderr, "output_init(): audio_out_fd=%d\n", audio_out_fd);
			}
		
		tmp = sample_size;
		ioctl(audio_out_fd, SNDCTL_DSP_SAMPLESIZE, &sample_size);
		if(tmp != sample_size)
			{
			fprintf(stderr, "output_init(): unable to set the sample size\n");
			return 0;
			}

		if(ioctl(audio_out_fd, SNDCTL_DSP_STEREO, &dsp_stereo) == -1)
			{
			fprintf(stderr, "output_init(): unable to set mono/stereo\n");

			return 0;
			}

		if(debug_flag)
			{
			fprintf(stderr, "output_init(): dsp_stereo=%d\n", dsp_stereo);

			if(dsp_stereo) fprintf (stderr, "output_init(): (stereo)\n");
			else fprintf (stderr, "output_init(): (mono)\n");
			}
	
		if(ioctl(audio_out_fd, SNDCTL_DSP_SPEED, &dsp_speed) == -1)
			{
			fprintf (stderr, "output_init(): unable to set audio speed\n");
			return 0;
			}

		if(debug_flag)
			{
			fprintf (stderr, "Speed %d Hz ", dsp_speed);
			if (sample_size != 8) 
				{
				fprintf(stderr, "output_init(): %d bits per sample\n", sample_size);
				}
		    }

		ioctl(audio_out_fd, SNDCTL_DSP_GETBLKSIZE, &outbuf_size);
		if(outbuf_size < 1)
			{
			fprintf(stderr, "output_init(): DSP_GETBLKSIZE failed, because ");

			perror ("GETBLKSIZE");
			return 0;
			}

		if(inbuf_size != outbuf_size)
			{
			if(debug_flag)
				{
				fprintf(stderr, "output_init(): inbuf_size=%d out outbuf_size=%d\n", inbuf_size, outbuf_size);
			
				outbuf_size = inbuf_size;
				} 		
			}

		} /* end if USE_OSS */

	} /* end if ! ouput_flag */

if(debug_flag)
	{
	fprintf(stderr, "output_init(): outbuf_size=%d\n", outbuf_size);
	}

//if( (!raw_output_flag) && ( (output_flag) || ( (internet_mode == INTERNET_CLIENT) && internet_stream_flag && local_output_flag) ) )
if( (! raw_output_flag) && output_flag)
	{

fprintf(stderr, "WAS write header\n");

/*
	if( ( (internet_mode != INTERNET_CLIENT) && use_input_file_flag) || \
	( (internet_mode == INTERNET_CLIENT) && internet_stream_flag && local_input_flag) )
*/
	if(use_input_file_flag) /* reading from file */
		{
		/* write a wave header same as input wave header */	
fprintf(stderr, "WAS copy in header\n");

		}
	else
		{	
		if(strcmp(output_filename, "-") != 0) /* no seek to start file possible if writing to stdout */
			{
fprintf(stderr, "WAS zero header\n");

			/* write a dummy wave header with all zeros, will be here again in exit via fseek with data_bytes_written */	
			memset(audiobuf, 0, header_size);
			}
		}

	a = fwrite(audiobuf, sizeof(char), header_size, ofptr);	
	if(a != header_size)
		{
		fprintf(stderr,\
		"xpequ: output_init(): could only write %d bytes of %d from output wave header, aborting.\n",\
		a, header_size);

		return 0;;		
		}
	}

return 1;
} /* end function output_init */


void *audio_routine()
{
int a, i, j, k;
size_t l = 0;
size_t tw;
int cm;
double da = 0.0;
double db = 0.0;
double dc = 0.0;
double dd = 0.0;
double de = 0.0;
double dm;
double dmax;
double old_da = 0.0;
int active = 0; // -Wall
int high_count, low_count;
unsigned char *pmbptr;
int bytes_per_sample;
float amp_float;
int bytes_written;
int do_output_flag;
int frames;
int eof_flag;
int bytes_read;
int frames_read = 0;
int num_frames = 0;
int triggered_flag;
double max_level;
double min_level;
double slice_level;

if(debug_flag)
	{
	fprintf(stderr, "audio_routine(): arg none\n");
	}

audio_running_flag = 1;

bytes_per_sample = sample_size / 8;

if(debug_flag)
	{
	fprintf(stderr,\
	"audiobuf=%p abuf_size=%d audio_in_fd=%d audiobuf=%p ifptr=%p use_input_file_flag=%d\n",\
	audiobuf, abuf_size, audio_in_fd, audiobuf, ifptr, use_input_file_flag);
	}

//while(1) sleep(1);

dmax = 0;
high_count = 0;
low_count = 0;
data_bytes_read = 0.0;
data_bytes_written = 0.0;
eof_flag = 0;
bytes_read = 0;
num_frames = abuf_size / ( (sample_size / 8) * audio_channels);
triggered_flag = 0;
k = 0;
j = 0;
slice_level = .5;
max_level = .5;
min_level = .5;
while(1)
	{
	if(stop_audio_flag)
		{
		/* test if need to write a modified wave header  */
		rewrite_wave_header();
	
		break;
		}

	if( (internet_mode == INTERNET_CLIENT) && internet_stream_flag ) /* read from net */
		{
		a = 0;
		while(1)
			{
			if(new_audio_flag)
				{
				new_audio_flag = 0;

				break;
				}

			usleep(10000); // 10ms

			a++;
			if(a == 2000) // 2s
				{
				rewrite_wave_header();
				break;
				}
			} /* end while wait for new audio */

		bytes_read = abuf_size;
		frames_read = bytes_read / ( (sample_size / 8) * audio_channels);

//fprintf(stderr, "WAS WHILE\n");

		}
	else
		{
		if(use_input_file_flag)
			{
			/* clear buffer, so it contains silence, and  equalization and fft on a partially filled buffer can be done */ 
			memset(audiobuf, abuf_size, 0);

			//size_t fread(void *ptr, size_t size, size_t nmemb, FILE *stream);
			bytes_read = fread(audiobuf, sizeof(char), abuf_size, ifptr);

			if(show_progres_flag && no_gui_flag)
				{
				dc = (data_bytes_read / (double)input_datalength) * 100.0;
				fprintf(stderr, "%.2f%%\r", dc);
				}

			if(bytes_read != abuf_size)
				{
				if(debug_flag)
					{
					fprintf(stderr, "xpequ: audio_routine(): fread() returned bytes_read=%d\n", bytes_read);
					}

				eof_flag = 1;
				}

			frames_read = bytes_read / ( (sample_size / 8) * audio_channels);
			} /* end if use_input_file_flag */
		else
			{	
			if(audio_mode == USE_ALSA)
				{
				frames_read = snd_pcm_readi(capture_handle, audiobuf, num_frames);
				if(frames_read < 0)
					{
					snd_pcm_prepare(capture_handle);

					fprintf(stderr, "audio_routine(): ALSA read buffer!!\n");

					frames_read = 0;
					}
				else
					{
					if(debug_flag)
						{
						fprintf(stderr, "audio_routine(): ALSA num_frames=%d frames_read=%d\n", num_frames, frames_read);
						}
	
					if(frames_read != num_frames)
						{
						if(debug_flag)
							{
							fprintf(stderr, "audio_routine(): ALSA read EOF\n");
							}

						if(use_input_file_flag)
							{
							eof_flag = 1;
							}
						}

					bytes_read = frames_read * (sample_size / 8) * audio_channels;
//					fprintf(stderr, "ALSA bytes_read=%d\n", bytes_read);
					}

				} /* end if USE_ALSA */
			else if(audio_mode == USE_OSS)
				{
				while(1)
					{
					bytes_read = 0;
					l = read(audio_in_fd, audiobuf + bytes_read, abuf_size - bytes_read);
					if(l == -1)
						{
						if(errno == EAGAIN)
							{
							fprintf(stderr, "audio_routine(): audio read(): EAGAIN\n");
							continue;
							}
	
						fprintf(stderr, "audio_routine(): audio read(): error return\n");
						perror("reason: ");

						audio_running_flag = 0;
						return 0;
						}

					if(l == 0) /* EOF */
						{
						eof_flag = 1;
						break;
						}

					bytes_read += l;				

					if(bytes_read == abuf_size) break;
					}

				} /* end end if USE_OSS */

			} /* end if read from device */

		} /* end if ! read from net */
	
	data_bytes_read += (double)bytes_read;

	if(internet_stream_flag)
		{	
		if(internet_mode == INTERNET_SERVER)
			{
			if(internet_client_sock != 0)
				{
				send_audio_buffer(internet_client_sock);
				}
			}
		}

	/* equalizer */

	if(equalizer_enable_flag)
		{
		frames = abuf_size / ( (sample_size / 8) * audio_channels);

		//int equalize(int16_t *data, int channels, int frames)
		equalize((int16_t*) audiobuf, audio_channels, frames);
		} /* end if equalizer_enable_flag */

	/* end equalizer */

	if(! no_gui_flag)
		{
		if(display_mode == DISPLAY_FFT)
			{
			/* update fft display */

			/* data to integer format */
			j = 0;
			for(i = 0; i < abuf_size; i += 2)
				{
				unsigned long p = 1 << 15;
				unsigned long ll;

				/* low byte first !!! */
				cm = audiobuf[i] + (256 * audiobuf[i + 1]);
				ll = cm;

				if(ll & p) /* sign bit */
					{
					a = (cm  & 0x7fff) -32768;
					}
				else /* positive */
					{
					a = cm;
					}

//				fprintf(stderr, "a=%d\n", a);

				wave[j].re = a;
				wave[j].im = 0.0;

				j++;
		
//				if(j > NUMSAMPLES) break;

				} /* end for i */           

//fprintf(stderr, "NUMSAMPLES=%d\n", NUMSAMPLES);  // 2048
//exit(1);

			/* perform FFT for channel data */
			/* the result is in 0 - 511 in wave */

			fft_window(fft, wave);
			fft_scale(wave, fft->bits);
			fft_compute(fft, wave);

			/* calculate outside loop */
			dd = (double)XYPLOT_WIDTH / 4.0;
			dc = (double)XYPLOT_WIDTH / ((double)XYPLOT_WIDTH - dd);
			de = log10((float)XYPLOT_HEIGHT);

			/* to display */
			for(i = 0; i < XYPLOT_WIDTH; i++)
				{
				amp_float = fft_amp(i, wave, fft->bits);

				if(fft_log_scale_flag)
					{
					/* normalize to 10000 */
					da = 10000.0 * ( (float)i / (float)XYPLOT_WIDTH ); /* 0 to 10000 */

					/* prevent log10(0) */
					if(da == 0) da = 1.0;
					db = log10(da); /* 1 to 4 */

					/* scale up to display size */
					da = db * dd; /* 128 to 512 */

					/* shift left to start display */
					da -= dd; /* 0 to 384 */

					/* rescale display to full width */
					da *= dc; /* 0 to 512 */
					}
				else
					{
					da = (float)i;
					}

				/* stay in array range */
				if(da < 0.0) da = 0.0;
				if(da > (XYPLOT_WIDTH - 1.0) ) da = XYPLOT_WIDTH - 1.0;

//fprintf(stderr, "i=%d da=%.2f db=%.2f a=%d amp_float=%.2f\n", i, da, db, a, amp_float);

				if(amp_float > 8192.0) amp_float = 8192.0;

				/* prevent log10(0) */
				if(amp_float <= 0.0) amp_float = 1.0;
				amp_float = (log10(amp_float) / de ) * (float)XYPLOT_HEIGHT;

				/* calibrate, to be done */
				amp_float /= 1.6;

				/* stay in array range */
				if(amp_float < 0.0) amp_float = 0;
				if(amp_float >= ( (float)XYPLOT_HEIGHT -1.0) ) amp_float = (float)XYPLOT_HEIGHT - 1.0; 

				fft_x[i] = da;
				fft_y[i] = amp_float;	
				}

//exit(1);		
			} /* end if display_mode is DISPLAY_FFT */
		else if(display_mode == DISPLAY_OSCILLOSCOPE)
			{
			/* update scope display */

			if(! scope_auto_trigger_flag)
				{
				slice_level = scope_trigger_level / 2.0;
				slice_level += .5;
				}

			/* data to integer format */
			for(i = 0; i < abuf_size; i += 2)
				{
				unsigned long p = 1 << 15;
				unsigned long ll;

				/* low byte first !!! */
				cm = audiobuf[i] + (256 * audiobuf[i + 1]);
				ll = cm;

				if(ll & p) /* sign bit */
					{
					a = (cm  & 0x7fff) -32768;
					}
				else /* positive */
					{
					a = cm;
					}
//				fprintf(stderr, "a=%d\n", a);

				da = (double)a;		/* -32768 to +32768 */

				da *= (float)scope_sensitivity;

				da += 32768.0;		/* 0 to 65535 */
				da /=  65536.0;		/* 0 to 1 */

//fprintf(stderr, "k=%d j=%d da=%.2f old_da=%.2f\n", k, j, da, old_da);

				if(scope_auto_trigger_flag)
					{
					/* scope_trigger_level is -1 to + 1*/
					dc = (double)scope_trigger_level + 1.0; /* 0 to 2 */
					dc /= 2.0; /* 0 to 1 */
					db = (7.0 / 8.0) * dc; /* 0 to 7/8 */

					if(da > max_level) max_level = da; /* .5 to 1 */
					if(da < min_level) min_level = da; /* .5 to 0 */

					if(scope_trigger_mode == SCOPE_TRIGGER_MODE_POS_EDGE)
						{
						slice_level = .5 + ( db * (max_level - .5) );
						}
					else if(scope_trigger_mode == SCOPE_TRIGGER_MODE_NEG_EDGE)
						{
						slice_level = .5 + ( db * (min_level - .5) );
						}	

					/* slowly drop back */
					if(max_level > 0.0) max_level -= .001;
					if(min_level < 1.0) min_level += .001;					
					} /* end if scope_auto_trigger_flag */

				if(! triggered_flag) /* wait for input to exceed trigger level */
					{
					/* pos edge triggered */
					if(scope_trigger_mode == SCOPE_TRIGGER_MODE_POS_EDGE)
						{
						if(da > old_da)
							{
							if(da > slice_level ) /* trigger level 0.0 to 1.0 */
								{
								triggered_flag = 1;

								j = 0; /* start at zero */
								}
							} 
						}
					else if(scope_trigger_mode == SCOPE_TRIGGER_MODE_NEG_EDGE) /* neg edge triggered */
						{
						if(da < old_da)
							{
							if(da < slice_level) /* trigger level 0.0 to 1.0 */
								{
								triggered_flag = 1;

								j = 0; /* start at zero */
								}
							} 
						}

					old_da = da;
					}
				else	/* have trigger */
					{
					old_da = da;

					/* scale to graphic size */
					da *= (float)XYPLOT_HEIGHT;
				
					scope_x[j] = (float)j;

					scope_y[j] = (float)da;

					/* run trace */
					if(k == 0)
						{
						j++; /* run trace */
						}
						
					if(j > XYPLOT_WIDTH - 1) /* if end of screen, trace to zero and wait for trigger */
						{
						triggered_flag = 0;

						j = 0;
						}
					} /* end if have trigger */

				k++;
				if(k >= scope_time_per_division_multiplier)
					{
					k = 0;
					}

				} /* end for i */           

			} /* end if display_mode is DISPLAY_OSCILLOSCOPE */

		} /* end if ! no_gui_flag */

	/* AGC */
	if(agc_flag)
		{
/* WARNING only 2 bytes per sample!! */

//		agc_level			/* 0.7, set in main, -3 dB */
//		agc_multiplier		/* 0-00,  set in main */

		/* test levels in audio buffer */ 
		pmbptr = audiobuf;
		for(i = 0; i < abuf_size / bytes_per_sample; i++)
      		{
			/* low byte first !!! */

			unsigned long p = 1 << 15;
			unsigned long ll;

			/* get main buffer as double */
			cm = *pmbptr + (256 * *(pmbptr + 1) );
			ll = cm;
			if(ll & p) /* sign bit */
				{
				ll = p - (ll ^ p);

				dm = -(double) ll / (double) p; 	
				}
			else /* positive */
				{
				dm = ll / ( (double) p );
				}

			/* range dm = -1 to +1 */

			/* modify amplitude */
			dm *= agc_multiplier;	/* -1 to +1 */

			/* level detect new value */			
			da = fabs(dm);			// 0-1
			if(da > agc_level)
				{
				/*scale back gain so samples fit */
				db = agc_level / da; /* always < 1.0 */
				if(agc_multiplier > 0.1) agc_multiplier *= db;

				agc_init_flag = 1;
				}

			if(! agc_init_flag)
				{
				/* at startup we start at gain 1x, fast increasing until we exceed the limit level for the first time */
				if(agc_multiplier < (double)AGC_MAX_GAIN) agc_multiplier += AGC_INIT_GAIN_INCREASE_SPEED;
				}
			else
				{
				/* slowly increase gain up to 34 dB (40dB = 100x) */
				/* 44100 samples / second, and 30 seconds, is 1323000 steps, so 50 / 323000 = .000037792 */
				/* a bit slower before we apply so much gain increase */
				if(agc_multiplier < (double)AGC_MAX_GAIN) agc_multiplier += AGC_GAIN_INCREASE_SPEED;
				}
				
			/* clip to prevent problems in 2 bytes integer */
			if(dm > 1.0) dm = 1.0;
			if(dm < -1.0) dm = -1.0;

			/* double data to output buffer */
			ll = floor(dm * p);
			if(dm < 0) ll += 2 * p;

			*pmbptr = ll % 256;
			*(pmbptr + 1) = ll / 256;

			/* increment buffer pointer */
			pmbptr += bytes_per_sample;
			} /* end for al values in audiobuf */

		if(debug_flag)
			{
/*
			fprintf(stderr,\
			"AGC: agc_multiplier=%.2f audio_current_level=%02f audio_peak_level=%.2f agc_level=%02f\n",\
			agc_multiplier, audio_current_level, audio_peak_level, agc_level);
*/
			}
		
		} /* end if agc_flag */
	/* end AGC */

	do_output_flag = 1;
	if(level_enabled_output_flag && (active_state == 0) ) do_output_flag = 0;

	if(do_output_flag)
		{
//fprintf(stderr, "WAS output_flag=%d local_output_flag=%d\n", output_flag, local_output_flag);

//		if( (output_flag) || ( (internet_mode == INTERNET_CLIENT) && internet_stream_flag && output_flag) )
		if(output_flag)
			{
			tw = fwrite(audiobuf, 1, bytes_read, ofptr);
			if(tw != bytes_read)
				{
				fprintf(stderr, "audio_routine(): could only write %d of %d bytes to file %s, aborting.\n", tw, bytes_read, output_filename);

				/* terminate thread */
				audio_running_flag = 0;

				return 0;
				}

			data_bytes_written += (double)tw;
			}
		else
			{
			if(audio_mode == USE_ALSA)
				{
//				fprintf(stderr, "ALSA frames_read=%d\n", frames_read);

				while(1)
					{
					a = snd_pcm_writei(playback_handle, audiobuf, frames_read);

					if(a < 0)
						{
						snd_pcm_prepare(playback_handle);		

						fprintf(stderr, "audio_routine(): ALSA write buffer!\n");
						}

//					if(a == frames_read) break;
					/* prevent hanging, sometimes for example only 2304 of 2308 frames are written at EOF */
					break;
					}

				} /* end if USE_ALSA */
			else if(audio_mode == USE_OSS)
				{
				unsigned char *abufptr;
				abufptr = audiobuf;
				bytes_written = 0;
				while(1)
					{
//					ssize_t write(int fd, const void *buf, size_t count);
					a = write(audio_out_fd, abufptr, bytes_read - bytes_written);
					if(a == -1)
						{
						if(errno == EAGAIN) continue;
						if(errno == EINTR) continue;

						if(debug_flag)
							{
							fprintf(stderr, "xpequ: audio_routine(): write() failed ");
							perror("because: ");
							}

						/* terminate thread */
						audio_running_flag = 0;
						return 0;
						}

					bytes_written += a;
					abufptr += a;
				
					if(bytes_written == bytes_read) break;
					}
				} /* end if USE_OSS */ 

			} /* end if output_flag */
		} /* end if do_output_flag */

	/* test levels in audio buffer */
	pmbptr = audiobuf;
	for(i = 0; i < abuf_size / bytes_per_sample; i++)
		{
		/* low byte first !!! */

		unsigned long p = 1 << 15;
		unsigned long ll;

		/* get main buffer as double */
		cm = *pmbptr + (256 * *(pmbptr + 1) );
		ll = cm;
		if(ll & p) /* sign bit */
			{
			ll = p - (ll ^ p);

			dm = -(double) ll / (double) p; 	
			}
		else /* positive */
			{
			dm = ll / ( (double) p );
			}

		/* range dm = -1 to +1 */
//		fprintf(stderr, "dm=%.10f\r", dm);
		
		da = fabs(dm);			// 0-1

		// avoid zero
		da += .00000000000001;
//		audio_current_level = 20 * log10(da / db);
		audio_current_level = 20 * log10(da / 1.0);

		if(audio_current_level > audio_peak_level)
			{
			audio_peak_level = audio_current_level;
			}
		/* speed at which audio peak meter goes to zero */ 
		if(audio_peak_level > minimum_db + VU_METER_DROP_BACK_SPEED) audio_peak_level -= VU_METER_DROP_BACK_SPEED;

		if(! active)
			{
			if(audio_current_level > threshold) if(high_count < high_samples) high_count++;
			if(high_count == high_samples)
				{
				low_count = 0;
				active = 1;
				active_state = 1;

				if(debug_flag)
					{
					fprintf(stderr, "active\n");
					}

				}
			}

		if(active)
			{
			if(audio_current_level < threshold)
				{
				if(low_count < low_samples) low_count++;
				}
			else low_count = 0;

			if(low_count == low_samples)
				{
				high_count = 0;
				active = 0;
				active_state = 0;

				if(debug_flag)
					{
					fprintf(stderr, "not active\n");
					}
				}
			}

		pmbptr += bytes_per_sample;
		} /* end for all samples in audio buffer */

	if(eof_flag)
		{
		fprintf(stderr, "xpequ: audio_routine(): EOF detected, exiting.\n");

		/* write last bit to output */
		fflush(ofptr);

		/* test if need to write a modified wave header  */
		rewrite_wave_header();
	
		ask_client_to_disconnect(internet_client_sock);

		/* exit OK */
		exit(0);
		} /* end if eof_flag */

	} /* end while read audio buffer */

audio_running_flag = 0;
return 0;
} /* end function audio_routine */


int stop_audio()
{
if(debug_flag)
	{
	fprintf(stderr, "stop_audio(): arg none, using audio_in_fd=%d audio_out_fd=%d\n", audio_in_fd, audio_out_fd);
	}

stop_audio_flag = 1;

while(audio_running_flag)
	{
	usleep(10000);
//	sleep(1);
	}

if(audio_mode == USE_ALSA)
	{
	if(capture_handle)
		{
//		snd_pcm_drain(capture_handle);
		snd_pcm_close(capture_handle);
		capture_handle = 0;
		}

	if(playback_handle)
		{
		snd_pcm_close(playback_handle);
		playback_handle = 0;
		}

	} /* end if USE_ALSA */
else if(audio_mode == USE_OSS)	
	{
	if(audio_in_fd > 0)
		{
		close(audio_in_fd);
		audio_in_fd = 0;	
		}

	if(audio_out_fd > 0)
		{
		close(audio_out_fd);
		audio_out_fd = 0;
		}
	} /* end if USE_OSS */

//if(internet_mode != INTERNET_CLIENT)
	{
	/* test if need to write modified wave header */
//	rewrite_wave_header();

	/* close if stdin or stdout? */
	if(use_input_file_flag) fclose(ifptr);
	if(output_flag) fclose(ofptr);
	}
	
return 1;
} /* end function stop_audio */


int start_audio()
{
if(debug_flag)
	{
	fprintf(stderr,\
	"start_audio(): using audio_input_device_name=%s(%p) audio_output_device_name=%s(%p)\n",\
	audio_input_device_name, audio_input_device_name, audio_output_device_name, audio_output_device_name); 
	}

stop_audio_flag = 0;
audio_running_flag = 0;

if(! input_init() )
	{
	/* allow changing in dsp */
	return 0;
	}

if(! output_init() )
	{
	/* allow changing out dsp */
	return 0;
	}

pthread_create(&audio_thread, NULL, &audio_routine, NULL);

if(debug_flag)
	{
	fprintf(stderr, "start_audio(): audio_started\n");
	}

return 1;
} /* end function start_audio */


int rewrite_wave_header()
{
int a;
int length_chunk;
int modus;
int bytes_per_second;
int bytes_per_sample;
int bits_per_sample;
uint32_t data_length;
uint32_t file_length;

if(debug_flag)
	{
	fprintf(stderr,\
	"rewrite_wave_header(): arg none,\n\
	using output_flag=%d raw_output_flag=%d output_filename=%s data_bytes_read=%.0f data_bytes_written=%.0f ofptr=%p\n",\
	output_flag, raw_output_flag, output_filename, data_bytes_read, data_bytes_written, ofptr);
	}

/* when not to write a wave header */

/* no output to file */
if(! output_flag) return 1;

/* raw output */
if(raw_output_flag) return 1;

/* if to stdout */
if(strcmp(output_filename, "-") == 0) return 1;

fprintf(stderr, "WAS rewrite wave header\n");

/* construct a wave header */

//char main_chunk[4];		/* +0	'RIFF' */
//uint32_t length;			/* +4	length of file */
//uint8_t chunk_type[4];	/* +8	'WAVE' */
//uint8_t sub_chunk[4];		/* +12	'fmt ' */
//uint32_t length_chunk;	/* +16	length sub_chunk, always 16 bytes */
//uint16_t format;			/* +20	always 1 = PCM-Code */
//uint16_t modus;			/* +22	1 = Mono, 2 = Stereo */
//uint32_t sample_fq;		/* +24	Sample Freq */
//uint32_t byte_p_sec;		/* +28	Data per sec */
//uint16_t byte_p_spl;		/* +32	bytes per sample, 1=8 bit, 2=16 bit (mono) 2=8 bit, 4=16 bit (stereo) */
//uint16_t bit_p_spl;		/* +34	bits per sample, 8, 12, 16 */
//uint8_t data_chunk[4];	/* +36	'data' */
//uint32_t data_length;		/* +40	length of data */
// total length = 44 bytes

length_chunk = 16;
modus = audio_channels;
bytes_per_second = dsp_speed * audio_channels * (sample_size / 8);
bits_per_sample = sample_size;
bytes_per_sample = sample_size / 8;

if(data_bytes_read > (double)0xffffffff)
	{
	data_length = (uint32_t)0xffffffff;
	}
else
	{
	data_length = (uint32_t)data_bytes_written;
	}

if( (data_length + (double)header_size) > ( (double)0xffffffff + (double)header_size) )
	{
	file_length = (uint32_t)(double)0xffffffff;
	}
else
	{
	file_length = (uint32_t)(data_length + header_size - 8);
	}		

//fprintf(stderr, "data_length=%d file_length=%d\n", data_length, file_length);

audiobuf[ 0] = 'R';
audiobuf[ 1] = 'I';
audiobuf[ 2] = 'F';
audiobuf[ 3] = 'F';

audiobuf[ 4] = file_length & 0xff;
audiobuf[ 5] = (file_length & 0xff00) >> 8;
audiobuf[ 6] = (file_length & 0xff0000) >> 16;
audiobuf[ 7] = (file_length & 0xff000000) >> 24;

audiobuf[ 8] = 'W';
audiobuf[ 9] = 'A';
audiobuf[10] = 'V';
audiobuf[11] = 'E';

audiobuf[12] = 'f';
audiobuf[13] = 'm';
audiobuf[14] = 't';
audiobuf[15] = ' ';

audiobuf[16] = length_chunk & 0xff;
audiobuf[17] = (length_chunk & 0xff00) >> 8;
audiobuf[18] = (length_chunk & 0xff0000) >> 16;
audiobuf[19] = (length_chunk & 0xff000000) >> 24;

audiobuf[20] = 1;
audiobuf[21] = 0;

audiobuf[22] = modus & 0xff;
audiobuf[23] = ( (modus) & 0xff00) >> 8;

audiobuf[24] = dsp_speed & 0xff;
audiobuf[25] = (dsp_speed & 0xff00) >> 8;
audiobuf[26] = (dsp_speed & 0xff0000) >> 16;
audiobuf[27] = (dsp_speed & 0xff000000) >> 24;

audiobuf[28] = bytes_per_second  & 0xff;
audiobuf[29] = (bytes_per_second & 0xff00) >> 8;
audiobuf[30] = (bytes_per_second & 0xff0000) >> 16;
audiobuf[31] = (bytes_per_second & 0xff000000) >> 24;

audiobuf[32] = bytes_per_sample & 0xff;
audiobuf[33] = (bytes_per_sample & 0xff00) >> 8;

audiobuf[34] = bits_per_sample & 0xff;
audiobuf[35] = (bits_per_sample & 0xff00) >> 8;

audiobuf[36] = 'd';
audiobuf[37] = 'a';
audiobuf[38] = 't';
audiobuf[39] = 'a';

audiobuf[40] = data_length & 0xff;
audiobuf[41] = (data_length & 0xff00) >> 8;
audiobuf[42] = (data_length & 0xff0000) >> 16;
audiobuf[43] = (data_length & 0xff000000) >> 24;

/* seek to start of file, overwrite header space there */
a = fseek(ofptr, 0, SEEK_SET);
if(a != 0)
	{
	fprintf(stderr, "xpequ: stop_audio(): fseek to start of file %s failed, aborting.\n", output_filename);

	return 0;
	}

/* write a wave header same as input wave header */	
a = fwrite(audiobuf, sizeof(char), header_size, ofptr);	
if(a != header_size)
	{
	fprintf(stderr,\
	"xpequ: rewrite_wave_header(): could only write %d bytes of %d from output wave header, aborting.\n",\
	a, header_size);

	return 0;		
	}

return 1;
} /* end function rewrite_wave_header */

