/*
xpequ copyright (c) 2007-always Jan Panteltje

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#include "xpequ.h"

FD_xpequ *fdui;

pthread_t colors_thread;
pthread_attr_t *attributes;

static int stop_colors_flag;
static FILE *fptr = 0;
static int red, green, blue;



void *colors_routine()
{
char temp[80];

if(debug_flag)
	{
	fprintf(stderr, "colors_routine(): arg none, using stop_colors_flag=%d\n", stop_colors_flag);
	}

colors_running_flag = 1;

/* test for user action */
while(1)
	{
	if(stop_colors_flag)
		{
		fclose(fptr);

		fptr = 0;

		colors_running_flag = 0;

		return 0;
		}	

	/* for RGB lights with col_pic */ 
	int pri_flag = 0;

	if(color_output_flag)
		{
		if(! fptr)
			{
			/* test if device exists, do not want to create a text file in /dev/ with "w" */
			fptr = fopen(serial_device_name, "r");
			if(! fptr)
				{
				if(no_gui_flag)
					{
					fprintf(stderr, "xpequ: colors_routine(): could not open serial device %s for read.\n", serial_device_name);
					}

				colors_error_mode = COLORS_ERROR_DEVICE;

				colors_running_flag = 0;

				return 0;
				}
			fclose(fptr);		

			fptr = fopen(serial_device_name, "w");
			if(! fptr)
				{
				if(no_gui_flag)
					{
					fprintf(stderr, "xpequ: colors_routine(): could not open serial device %s for write.\n", serial_device_name);
					}

				colors_error_mode = COLORS_ERROR_DEVICE;

				colors_running_flag = 0;

				return 0;
				}

			// int set_port_parameters(int fd, int baudrate, char *device)
			if(! set_port_parameters(fileno(fptr), baudrate, serial_device_name) )
				{
				if(no_gui_flag)
					{
					fprintf(stderr, "xpequ: idle_cb(): could not set baudrate %d\n", baudrate);
					}
	
				colors_error_mode = COLORS_ERROR_PORT_PARAMETERS;

				colors_running_flag = 0;

				return 0;
				}
			} /* end if ! fptr */

		if(pri_flag)
			{
			fprintf(stderr, "serial_device_name=%s fptr=%p, fileno(fptr)=%d baudrate=%d\n", serial_device_name, fptr, fileno(fptr), baudrate );
			}

		red = average_red;		// set in equaliser.c
		green = average_green;
		blue = average_blue;

		sprintf(temp, "R%d\n", red);
		if(pri_flag) fprintf(stderr, "sending temp=%s", temp);
		//size_t fwrite(const void *ptr, size_t size, size_t nmemb, FILE *stream);
		fwrite(temp, 1, strlen(temp) + 1, fptr);	

		sprintf(temp, "G%d\n", green);
		if(pri_flag) fprintf(stderr, "sending temp=%s", temp);
		fwrite(temp, 1, strlen(temp) + 1, fptr);	

		sprintf(temp, "B%d\n", blue);
		if(pri_flag) fprintf(stderr, "sending temp=%s\n", temp);
		fwrite(temp, 1, strlen(temp) + 1, fptr);	

		times_here_red = 0;
		avrg_red = 0.0;
		
		times_here_green = 0;
		avrg_green = 0.0;
		
		times_here_blue = 0;
		avrg_blue = 0.0;

		/*
		The PIC runs at 19200 Baud, so it can handle, with 1 start -, 8 data -, and 1 stop bit, 1920 bytes per second, or .52 ms per byte.
		There are 3 bytes needed for every color 'RXLF' so 9 bytes to set the color, 9 bytes will take 9 x .52 = 4.6 ms.
		For the lowest audio frequency, say 100 Hz for the 125 Hz filter, we need 10 mS time to average a full period.
		For a 20 Hz signal we would need 50 mS.
		If we wait 4.6 + 30 mS then the lowest frequency of which we can average a complete period is 1 / 34.6 = 28.9 Hz.
		At 28.9 Hz the output fromn the 125 Hz filter will be very low.		
		So 30 ms extra delay seems a good compromise.
		colo_pic will flash at 28.9 Hz, due to the interrupts changing the timing of the software PWM generation in the main routine.
		This could be fixed by using 3 PICs with hardware PWM, listening to the same RS232.
		*/
		/* 30 ms, empirically for best result */
		usleep(30000);		

		} /* end if color_output_flag */
	} /* end wile do colors */

/* never here */

colors_running_flag = 0;

return 0;
} /* end function colors_routine */



int stop_colors()
{
if(debug_flag)
	{
	fprintf(stderr, "stop_colors(): arg none.\n");
	}

stop_colors_flag = 1;

while(colors_running_flag)
	{
	usleep(10000);
	}


colors_error_mode = COLORS_ERROR_NONE;

return 1;
} /* end function stop_colors */



int start_colors()
{
if(debug_flag)
	{
	fprintf(stderr,\
	"start_colors(): using serial device=%s baudrate=%d\n", serial_device_name, baudrate);
	}

colors_error_mode = COLORS_ERROR_NONE;

stop_colors_flag = 0;
colors_running_flag = 0;

pthread_create(&colors_thread, NULL, &colors_routine, NULL);

if(debug_flag)
	{
	fprintf(stderr, "start_colors(): colors_started\n");
	}

return 1;
} /* end function start_colors */



