
/*
xpequ copyright (c) 2007-always Jan Panteltje

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/



#include "forms.h"
#include "xpequ.h"

extern FD_xpequ *fdui;



int print_usage()
{
fprintf(stderr, "\n\
%s by Jan Panteltje (c) 2007-always.\n\
Usage:               xpequ [-a] [-b bits_per_sample] [-c] [-d] [-e equalizer_settings_filename] [-h] [-i input_filename]\n\
  [-m] [-n server_name] [-o output_filename] [-p port] [-s] [-t output] [-u input] [-v] [-x] [-y] [-z sample_speed]\n\
-a                   no gui, use in case -i -o from file to file, for example in batch mode from a script.\n\
-b bits_per_sample   bits per sample of input stream, if specified xpequ will treat stdin as stream, not a file.\n\
-c channels          number of channels in input stream, if specified xpequ will treat stdin as stream, not a file.\n\
-d                   show progres in %% in -i -a.\n\
-e filename          save and load equalizer settings from filename (default ~/.xpequ/setup/inputfilename.equalizer).\n\
-g serial_device     serial device for lightshow output to col_pic, default /dev/ttyUSB0.\n\
-h                   help (this help).\n\
-i filename          input wave filename, use '-' for stdin, sets input dsp select button inactive.\n\
-k baudrate          baudrate for serial device, default 19200.\n\
-l                   lightshow output via specified serial device to 'col_pic'.\n\
-m                   use the cmix mixer program to set mic 100%%, igain 50%%, vol 100%%, and igain as source.\n\
-n server_name       name of the server to connect to in remote -y mode, default localhost.\n\
-o filename          records output to filename, use '-' for stdout. sets output dsp select button inactive, use with lame to make mp3.\n\
-p port              internet port for remote control, default 6668, use in -x or -y.\n\
-s                   raw output in case -o flag.\n\
-t output            selects ouput name, /dev/dspX selects OSS, hw:X,Y or plughw:X,Y selects ALSA.\n\
-u input             selects input name, /dev/dspX selects OSS, hw:X,Y or plughw:X,Y selects ALSA.\n\
-v                   debug (prints functions and arguments).\n\
-x                   monitor ~/.xpequ/setup/equalizer_settings in -a, so a second xpequ can set the equalizer.\n\
-y                   control client, use with -x -i -a for the other equalizer.\n\
-z sample_speed      sample speed of input stream, if specified xpequ will treat stdin as stream, not a file.\n\
\n\
Examples:\n\
Play a wave file, select a dsp device in the GUI:\n\
 xpequ -i file.wav\n\
Play a mp3 via /dev/dspX (select in GUI), and use the equalizer, with FFT display:\n\
 mpg123 -w - music.mp3 | xpequ -i -\n\
Stream /dev/dsp0 via lame mp3 encoder to create a mp3 file (optional level activated):\n\
 xpequ -s -o - | lame -r -s 44100 -m s -x - out.mp3\n\
Stream /dev/dsp0 via equalizer to stdout and then play via sox on dsp1:\n\
 xpequ -s -o - | sox -r 44100 -w -c 2 -s -t raw - -r 44100 -w -c 2 -s -t ossdsp /dev/dsp1\n\
Read a .wav file via the equalizer to a new wave file without GUI:\n\
 xpequ  -a -i sweep.wav -o sweep_corrected.wav\n\
Play a stereo .wav file via the equalizer using streaming and sox:\n\
 xpequ  -i sweep.wav -s -o - | sox -r 44100 -w -c 2 -s -t raw - -r 44100 -w -c 2 -s -t ossdsp /dev/dsp1\n\
Select best equalizer settings for wave file, and save on exit to samba.wav.equal,\n\
on restart the specified equalizer setting will be used again, so each file can have its own settings:\n\
 xpequ -e samba.wav.equal -i samba.wav -o samba_corrected.wav\n\
Record a wave file from the soundcard output (use 'E' button to stop):\n\
 xpequ -o music.wav\n\
It is possibe to run one xpequ with or without GUI, and an other xpequ to control it:\n\
controlled:\n\
 xpequ -x -i drums.wav -t hw:0,0 -p 6666\n\
controller:\n\
 xpequ -y -p 6666 -n localhost\n\
Stream to input, write to ALSA hw:1,0:\n\
 cat music.raw | xpequ -i - -z 44100 -c 2 -b 16 -t hw:1,0\n\
Drive 'disco lights' RGB LED strips via col_pic, using a USB to serial adapter on /dev/ttyUSB0, with 19200 Baud, to /dev/dsp1:\n\
xpequ  -g /dev/ttyUSB0 -k 19200 -l -i kaboom_kaboom.wav -s -o - | sox 2> /dev/zero -r 44100 -w -c 2 -s -t raw - -r 44100 -w -c 2 -s -t ossdsp /dev/dsp1\n\
\n",\
PROG_VERSION);

return 1;
} /* end function print_usage */


int main(int argc, char *argv[])
{
FD_xpequ *fd_xpequ;
FL_IOPT fl_iopt;
char *ptr;
int a;
Pixmap p, mask;
char temp[READSIZE];
unsigned int w, h;
FILE *fptr;
int have_sample_size_flag;
int have_channels_flag;
int have_dsp_speed_flag;

/* get user info */
userinfo = getpwuid(getuid() );

/* get home directory */ 
home_dir = (char *)strsave(userinfo -> pw_dir);
if(! home_dir)
	{
	fprintf(stderr, "xpequ: main(): cannot allocate space for home_dir, aborting.\n");
	exit(1);
	}

user_name = (char *)strsave(userinfo -> pw_name);
if(! user_name)
	{
	fprintf(stderr, "xpequ: main(): cannot allocate space for user_name, aborting.\n");
	exit(1);
	}

/* defaults */
debug_flag = 0;
output_flag = 0;
raw_output_flag = 0;
dsp_speed = DEFAULT_DSP_SPEED;
dsp_stereo = 0; // mono
sample_size = 16;
threshold = DEFAULT_THRESHOLD;
on_milli_seconds = DEFAULT_ON_TIME;
off_milli_seconds = DEFAULT_OFF_TIME;
set_mixer_flag = 0;
audio_input_device_number = DEFAULT_AUDIO_INPUT_DEVICE_NUMBER;
audio_output_device_number = DEFAULT_AUDIO_OUTPUT_DEVICE_NUMBER;
use_input_file_flag = 0;
no_gui_flag = 0;
show_progres_flag = 0;
audio_mode = USE_ALSA;
command_line_input_device_specified_flag = 0;
command_line_output_device_specified_flag = 0;
display_mode = DISPLAY_FFT;
scope_trigger_mode = SCOPE_TRIGGER_MODE_POS_EDGE;
scope_trigger_level = 0.0;
scope_time_per_division_multiplier = SCOPE_MIN_TIME_PER_DIVISION_MULTIPLIER;  
scope_auto_trigger_flag = 0;
scope_sensitivity = SCOPE_MIN_SENSITIVITY;
agc_init_flag = 0;
agc_level = 0.7;
agc_multiplier = 1.0;
input_stream_flag = 0;
have_sample_size_flag = 0;
have_channels_flag = 0;
have_dsp_speed_flag = 0;
internet_port = 6668;
internet_mode = INTERNET_OFF;
int command_line_color_output_flag;

/* for lightshow output via serial port to col_pic */
command_line_color_output_flag = 0;
baudrate = DEFAULT_BAUDRATE;

internet_server_name = strsave("localhost");
if(! internet_server_name)
	{
	fprintf(stderr, "xpequ: main(): cannot allocate space for internet_server_name, aborting.\n");

	exit(1);
	}	

input_filename = strsave("device");
if(! input_filename)
	{
	fprintf(stderr, "xpequ: main(): could not allocate space for input_filename, aborting.\n");

	exit(1);
	}


/* get old serial device setting if any */
ptr = load_setting("serial_device_name");
if(ptr)
	{
	serial_device_name = strsave(ptr);
	}
else
	{
	serial_device_name = strsave(DEFAULT_SERIAL_DEVICE_NAME);
	}

if(! serial_device_name)
	{
	fprintf(stderr, "xpequ: main(): could not allocate space for serial_device_name, aborting.\n");

	exit(1); 
	}

	
/* end defaults */

sprintf(temp, "%s/.xpequ/setup/equalizer_settings", home_dir);
equalizer_filename = strsave(temp);
if(! equalizer_filename)
	{
	fprintf(stderr, "xpequ: main(): could not allocate space for equalizer_filename, aborting.\n");
	
	exit(1);
	}

colors_error_mode = COLORS_ERROR_NONE;

update_gui_flag = 1;

/* end defaults */

/* set statptr for idle_cb */
statptr = (struct stat*) malloc(sizeof(struct stat) );
if(! statptr)
	{
	fprintf(stderr, "xpequ: main(): malloc() struc stat failed, aborting.\n");

	exit(1);
	}

/* set all time vars */
//ogettime(weekday, month, &monthday, &hour, &minute, &second, &year);

while(1)
	{
	a = getopt(argc, argv, "ab:c:de:g:hi:k:lmn:o:p:st:u:vxyz:");
	if(a == -1) break;

	if(debug_flag)
		{
		fprintf(stderr, "getopt() returned a=%d(%c)\n", a, a);
		}

	switch(a)
		{
		case 'a':	// no GUI
			no_gui_flag = 1;
			break;
		case 'b':	// bits per sample
			sample_size = atoi(optarg);
			have_sample_size_flag = 1;
			input_stream_flag = 1;
			break;
		case 'c':	// channels
			audio_channels = atoi(optarg);
			have_channels_flag = 1;
			input_stream_flag = 1;
			break;
		case 'd':
			show_progres_flag = 1;
			break;
		case 'e':
			free(equalizer_filename);
			equalizer_filename = strsave(optarg);
			if(! equalizer_filename)
				{
				fprintf(stderr, "xpequ: main(): '-e' could not allocate space for equalizer_filename, aborting.\n");
	
				exit(1);
				}

			break;
		case 'g': // serial device 
			free(serial_device_name);
			serial_device_name = strsave(optarg);
			if(! serial_device_name)
				{
				fprintf(stderr, "xpequ: main(): '-g' could not allocate space for serial_device_name, aborting.\n");

				exit(1);
				}

			save_setting("serial_device_name", serial_device_name);
			break;
		case 'h': // help
			print_usage();
			exit(1);
			break;
		case 'i':	// input filename
			if(input_filename) free(input_filename);
			input_filename = strsave(optarg);
			if(! input_filename)
				{
				fprintf(stderr, "xpequ: main(): could not allocate space for input_filename, aborting.\n");

				exit(1);
				}

			use_input_file_flag = 1;			
			break;
		case 'k': // baudrate serial device
			baudrate = atoi(optarg);
			break;
		case 'l': // lightshow output via serial port to col_pic.
			command_line_color_output_flag = 1;
			break;
		case 'm': // use cmix
			set_mixer_flag = 1;
			break;
		case 'n':
			free(internet_server_name);
			internet_server_name = strsave(optarg);
			if(! internet_server_name)
				{
				fprintf(stderr, "xpequ: main(): could not allocate space for internet_server_name, aborting.\n");

				exit(1);
				}
			break;
		case 'o': // PCM to stdout
			output_filename = strsave(optarg);
			if(! output_filename)
				{
				fprintf(stderr, "xpequ: main(): could not allocate space for output_filename, aborting.\n");

				exit(1);
				}
			output_flag = 1;
			break;
		case 'p':
			internet_port = atoi(optarg);
			break;
		case 's':
			raw_output_flag = 1;
			break;
		case 't': // output name
			audio_output_device_name = strsave(optarg);
			if(! audio_output_device_name)
				{
				fprintf(stderr, "xpequ: main(): could not set set_audio_output_device_name, aborting.\n");

				exit(1);
				}

			command_line_output_device_specified_flag = 1;
			break;
		case 'u': // input name
			audio_input_device_name = strsave(optarg);
			if(! audio_input_device_name)
				{
				fprintf(stderr, "xpequ: main(): could not set set_audio_input_device_name, aborting.\n");

				exit(1);
				}

			command_line_input_device_specified_flag = 1;
			break;
		case 'v': // debug
			debug_flag = 1;
			break;
		case 'x': // use with -a as remote controlled xpequ
			internet_mode = INTERNET_SERVER;
			break;
		case 'y': // use xpequ as remote control for one with -a 
			internet_mode = INTERNET_CLIENT;
			break;
		case 'z':	// sample speed
			dsp_speed = atoi(optarg);
			have_dsp_speed_flag = 1;
			input_stream_flag = 1;
			break;
        case -1:
        	break;
		case '?':
			if (isprint(optopt) )
 				{
 				fprintf(stderr, "Unknown option `-%c'.\n", optopt);
 				}
			else
				{
				fprintf(stderr, "Unknown option character `\\x%x'.\n",\
				optopt);
				}
			print_usage();

			exit(1);
			break;			
		default:
			print_usage();

			exit(1);
			break;
		}/* end switch a */
	}/* end while getopt() */




/*
fprintf(stderr, "use_input_file_flag=%d output_flag=%d raw_output_flag=%d\n",\
use_input_file_flag, output_flag, raw_output_flag);
*/

/* sanity checks */
if(raw_output_flag)
	{
	if(! output_flag)
		{
		fprintf(stderr, "xequ: main(): -s flag without -o flag specified, aborting.\n");

		exit(1);	
		}
	}

if( (!raw_output_flag) && output_flag && ( strcmp(output_filename, "-") == 0) )
	{
	fprintf(stderr, "xpequ: main(): cannot write wave header to stdout stream, use a real output filename or -s for raw mode, aborting.\n");
	
	exit(1);
	}
	
if(input_stream_flag)
	{
	a = 1;
	if(! have_channels_flag)
		{
		fprintf(stderr,\
		"xpequ: main(): input streaming requested, but no number of channels specified, use -c command line option, aborting.\n");

		a = 0;
		}

	if(! have_dsp_speed_flag)
		{
		fprintf(stderr,\
		"xpequ: main(): input streaming requested, but no sample speed specified, use -z command line option, aborting.\n");

		a = 0;
		}	

	if(! have_sample_size_flag)
		{	
		fprintf(stderr,\
		"xpequ: main(): input streaming requested, but no sample_size specified, use -b command line option, aborting.\n");

		a = 0;	
		}

	if(sample_size != 16)
		{
		fprintf(stderr, "xpequ: main(): only 16 bits per sample supported, aborting.\n");
		
		a = 0;
		}

	if(a == 0)
		{
		exit(1);
		}
	} /* end if input_stream_flag */

if(internet_mode == INTERNET_CLIENT)
	{
	if(use_input_file_flag)
		{
		fprintf(stderr, "xpequ: main(): no input file (-i) allowed in internet client mode (-y), aborting.\n");
		
		exit(1);
		}
	}

/* end sanity check */

inbuffer = (char *)malloc(INTERNET_BUFFER_SIZE);
if(! inbuffer)
	{
	fprintf(stderr, "xpequ: main(): cannoit allocate space for inbuffer, aborting.\n");
	
	exit(1);	
	}

/* test if we couple equalizer settings to file name */
/* default */
a = 1;
/* when not */
if(internet_mode == INTERNET_SERVER) a = 0;							// not if remote controlled (-x).
if(internet_mode == INTERNET_CLIENT) a = 0;							// not if remote controller (-y).
if(! use_input_file_flag) a = 0;									// not if not read from file (! -i).

//ptr = strstr(input_filename, ".wav");								
//if(! ptr) a = 0;
//if(ptr != (input_filename + strlen(input_filename) + 4) ) a = 0;	// not if filename does not end in .wav  
//if(strcmp(input_filename, "-") == 0) a = 0;						// not if read from stdin.
/* overrule conditions */

if(a)
	{
	free(equalizer_filename);
	/* keep only the real filename, not the path */
	ptr = strrchr(input_filename, '/');
	if(! ptr)
		{
		sprintf(temp, "%s/.xpequ/%s.equalizer", home_dir, input_filename);
		}
	else
		{
		sprintf(temp, "%s/.xpequ/%s.equalizer", home_dir, ptr + 1);
		}	
	equalizer_filename = strsave(temp);
	if(! equalizer_filename)
		{
		fprintf(stderr, "xpequ: main(): '-e' could not allocate space for equalizer_filename, aborting.\n");
	
		exit(1);
		}
	} /* end if a */

if(debug_flag)
	{
	fprintf(stderr, "equalizer_filename=%s\n", equalizer_filename);
	}

/* against flashing */
/* set double buffering */
fl_iopt.doubleBuffer = 1;
fl_set_defaults(FL_PDDouble, &fl_iopt);
                   
fl_initialize(&argc, argv, 0, 0, 0);

fd_xpequ = create_form_xpequ();

/* set the icon */
sprintf(temp, "%s/.xpequ/xpequ.xpm", home_dir);
fptr = fopen(temp, "r");
if(fptr)
	{
	fclose(fptr);

	p = fl_read_pixmapfile(fl_root, temp, &w, &h, &mask, 0, 0, 0);

	fl_set_form_icon(fdui -> xpequ, p, mask);                 
	}
else
	{
	fprintf(stderr, "no icon %s found\n", temp);
	}

if(! load_audio_settings() )
	{
	fprintf(stderr, "xpequ: main(): load_audio_settings() failed, aborting.\n");	
	
	exit(1);
	}

if(command_line_color_output_flag) color_output_flag = 1;
if(color_output_flag)
	{
	start_colors();

	fl_set_button(fdui -> color_output_button, color_output_flag);
	}

if(command_line_input_device_specified_flag)
	{
	if(strstr(audio_input_device_name, "/dev/dsp") )
		{
		a = sscanf(audio_input_device_name, "/dev/dsp%d", &audio_input_device_number);
		if(a != 1)
			{
			fprintf(stderr, "xpequ:	main(): cannot parse audio_input_device_name %s, aborting.\n", audio_input_device_name);

			exit(1);
			}

		if(audio_mode != USE_OSS)
			{
			if(debug_flag)
				{
				fprintf(stderr, "xpequ: main(): overriding GUI audio mode ALSA.\n");
				}
			}

		audio_mode = USE_OSS;
		}
	else
		{
		a = sscanf(audio_input_device_name, "hw:%d,%d", &audio_input_device_number, &audio_input_device_sub_number);
		if(a != 2)
			{
			a = sscanf(audio_input_device_name, "plughw:%d,%d", &audio_input_device_number, &audio_input_device_sub_number);

			if(a != 2)
				{
				fprintf(stderr, "xpequ:	main(): cannot parse audio_input_device_name %s, aborting.\n", audio_input_device_name);

				exit(1);			
				}
			}

		if(audio_mode != USE_ALSA)
			{
			if(debug_flag)
				{
				fprintf(stderr, "xpequ: main(): overriding GUI audio mode OSS.\n");
				}
			}

		audio_mode = USE_ALSA;
		}

//	set_audio_input_device_name(audio_input_device_number, audio_input_device_sub_number);
	}

if(command_line_output_device_specified_flag)
	{
	if(strstr(audio_output_device_name, "/dev/dsp") )
		{
		a = sscanf(audio_output_device_name, "/dev/dsp%d", &audio_output_device_number );
		if(a != 1)
			{
			fprintf(stderr, "xpequ:	main(): cannot parse audio_output_device_name %s, aborting.\n", audio_output_device_name);
			exit(1);			
			}

		if(command_line_input_device_specified_flag && (audio_mode != USE_OSS) )
			{
			fprintf(stderr, "xpequ: main(): audio input and output device must both be OSS or ALSA, aborting.\n");

			exit(1);
			}
		else
			{
			if(audio_mode != USE_OSS)
				{
				if(debug_flag)
					{
					fprintf(stderr, "xpequ: main(): overriding GUI audio mode ALSA.\n");
					}
				}
			}

		audio_mode = USE_OSS;
		}
	else
		{
		a = sscanf(audio_output_device_name, "hw:%d,%d", &audio_output_device_number, &audio_output_device_sub_number);
		if(a != 2)
			{
			a = sscanf(audio_output_device_name, "plughw:%d,%d", &audio_output_device_number, &audio_output_device_sub_number);

			if(a != 2)
				{
				fprintf(stderr, "xpequ: main(): cannot parse audio_output_device_name %s, aborting.\n", audio_output_device_name);
				exit(1);			
				}
			}

		if(command_line_input_device_specified_flag && (audio_mode != USE_ALSA) )
			{
			fprintf(stderr, "xpequ: main(): audio input and output device must both be OSS or ALSA, aborting.\n");

			exit(1);
			}
		else
			{
			if(audio_mode != USE_ALSA)
				{
				if(debug_flag)
					{
					fprintf(stderr, "xpequ: main(): overriding GUI audio mode OSS.\n");
					}
				}
			}

		audio_mode = USE_ALSA;
		}

//	set_audio_output_device_name(audio_output_device_number, audio_output_device_sub_number);
	} /* end if command_line_output_device_specified_flag */

/* these functions automatically sequence through devices and subdevices until input and output are not the same */
set_audio_input_device_name(audio_input_device_number, audio_input_device_sub_number);   
set_audio_output_device_name(audio_output_device_number, audio_output_device_sub_number);

#ifdef TIMING_CHECKS
start_timeval = malloc(sizeof(struct timeval) );
if(! start_timeval)
	{
	fprintf(sterr, "could not allocate space for start_timeval\n");

	exit(1);
	}

current_timeval = malloc(sizeof(struct timeval) );
if(! current_timeval)
	{
	fprintf(stderr, "could not allocate space for current_timeval\n");

	exit(1);
	}
#endif // TIMING_CHECKS

fft = fft_new(FFT_BITS);

header_size = AUDIO_HEADER_SIZE;
abuf_size = AUDIO_BUFFER_SIZE;	
audiobuf = malloc(abuf_size);
if(! audiobuf)
	{
	fprintf(stderr, "xpequ: main(): could not allocate space for audiobuf, aborting.\n");
	
	exit(1);
	}

in_header = (wave_header *) malloc(header_size);
if(! in_header)
	{
	fprintf(stderr, "xpequ: main(): could not allocate space for in_header, aborting.\n");

	exit(1);
	}

if(! no_gui_flag)
	{
	/* idle_cb uses this! */
	fft_x = malloc(XYPLOT_WIDTH * sizeof(float));
	if(! fft_x)
		{
		fprintf(stderr, "input_init(): could not malloc fft_x, aborting.\n");	

		exit(1);
		}

	fft_y = malloc(XYPLOT_WIDTH * sizeof(float));
	if(! fft_y)
		{
		fprintf(stderr, "input_init(): could not malloc fft_y, aborting.\n");	

		exit(1);
		}

	/* idle_cb uses this! */
	scope_x = malloc(XYPLOT_WIDTH * sizeof(float));
	if(! scope_x)
		{
		fprintf(stderr, "input_init(): could not malloc scope_x, aborting.\n");	

		exit(1);
		}

	scope_y = malloc(XYPLOT_WIDTH * sizeof(float));
	if(! scope_y)
		{
		fprintf(stderr, "input_init(): could not malloc scope_y, aborting.\n");	

		exit(1);
		}

	scope_measure_x = malloc(XYPLOT_WIDTH * sizeof(float));
	if(! scope_measure_x)
		{
		fprintf(stderr, "input_init(): could not malloc scope_measure_x, aborting.\n");	

		exit(1);
		}

	scope_measure_y = malloc(XYPLOT_WIDTH * sizeof(float));
	if(! scope_measure_y)
		{
		fprintf(stderr, "input_init(): could not malloc scope_measure_y, aborting.\n");	

		exit(1);
		}
	} /* end if ! no_gui_flag */

/* equalizer */
equalizer_init();
load_equalizer(equalizer_filename);

if(internet_mode != INTERNET_CLIENT)
	{
	if(! start_audio() )
		{
		fprintf(stderr, "xpequ: main(): could not initilize audio thread, try a different dsp input device.\n");

/*
		fl_show_alert(\
		"could not initialize audio thread", audio_input_device, "try a different input device", 0);
*/

		/* allow user to select an other dsp input device */
		}
	} /* end if internet_mode != INTERNET_CLIENT */		

minimum_db = 20 * log10(1.0 / ( (1 << 15) / 2) );
if(debug_flag)
	{
	fprintf(stderr, "minimum_db=%.2f\n", minimum_db);
	}

fl_set_slider_bounds(fd_xpequ -> threshold_slider, 0, minimum_db);
fl_set_slider_bounds(fd_xpequ -> audio_peak_indicator_slider, 0, minimum_db);

milli_seconds_per_sample = 1000.0 / (double)dsp_speed;

if(debug_flag)
	{
	fprintf(stderr, "milli_seconds_per_sample=%.2f\n", milli_seconds_per_sample);
	fprintf(stderr, "on_milli_seconds=%.2f off_milli_seconds=%.2f\n", on_milli_seconds, off_milli_seconds);
	fprintf(stderr, "high_samples=%d low_samples=%d\n", high_samples, low_samples);
	}

fl_set_xyplot_xtics(fdui -> fft_display, -1, -1);
fl_set_xyplot_ytics(fdui -> fft_display, -1, -1);

fl_set_xyplot_xbounds(fdui -> fft_display, 1.0, (float)XYPLOT_WIDTH);
fl_set_xyplot_ybounds(fdui -> fft_display, 1.0, (float)XYPLOT_HEIGHT);

fl_set_xyplot_xtics(fdui -> scope_display, -1, -1);
fl_set_xyplot_ytics(fdui -> scope_display, -1, -1);

fl_set_xyplot_xbounds(fdui -> scope_display, 1.0, (float)XYPLOT_WIDTH);
fl_set_xyplot_ybounds(fdui -> scope_display, 1.0, (float)XYPLOT_HEIGHT);

//(int)high_samples = (double) on_milli_seconds / (double) milli_seconds_per_sample;
high_samples = on_milli_seconds / milli_seconds_per_sample;

//(int)low_samples = (double) off_milli_seconds / (double) milli_seconds_per_sample;
low_samples = off_milli_seconds / milli_seconds_per_sample;

/* want to show and enable when */

if(use_input_file_flag)
	{
	fl_hide_object(fdui -> audio_input_device_select_button);
	fl_hide_object(fdui -> audio_input_text);
	}

if(output_flag)
	{
	fl_hide_object(fdui -> audio_output_device_select_button);	
	fl_hide_object(fdui -> audio_output_text);
	}

if(output_flag && use_input_file_flag)
	{
	fl_hide_object(fdui -> audio_mode_select_button);
	}

if(! no_gui_flag)
	{
	/* show the first form */
	fl_show_form(fd_xpequ -> xpequ, FL_PLACE_FREE, FL_FULLBORDER, PROG_VERSION);

	/* let's face it, xforms does not see -geometry */
	}

if( (internet_mode == INTERNET_SERVER) || (internet_mode == INTERNET_CLIENT) )
	{
	if(! start_internet() )
		{
		fprintf(stderr, "xpequ: main(): could not initialize internet, no remote control possible.\n");
		}

	if(internet_mode == INTERNET_CLIENT)
		{
		fl_set_object_label(fdui -> internet_server_name_indicator, internet_server_name);		
		fl_show_object(fdui -> internet_server_name_indicator);
		}
	fl_show_object(fdui -> internet_stream_button);
	}
else
	{
	fl_hide_object(fdui -> internet_server_name_indicator);		
	fl_hide_object(fdui -> internet_stream_button);
	}
	
fl_set_object_label(fdui -> serial_device_select_button, serial_device_name);

if(internet_mode == INTERNET_CLIENT)
	{
	fl_hide_object(fdui -> serial_device_select_button);
	fl_hide_object(fdui -> color_output_button);
	}

fl_do_forms();

exit(0);
} /* end function main */


